/** 

@file pex_interface.i 

SWIG interface for example plugin. This is a particulary simple
case, since it defines only one additional function and no
data-types. Thus, the example plugin is regarded a good 
starting point for getting into lua integration of faudes
plugins. 

See the luabindings README for more documentation. 
 
**/


/*
**************************************************
**************************************************
**************************************************

Example interface preface

**************************************************
**************************************************
**************************************************
*/

// Set SWIG module name
// Note: must match libFAUDES plugin name
%module example

// Load std SWIG includes
%include <std_string.i>
%include <std_except.i>

// Import core faudes interface
// Note: this includes swig macros 
%import "corefaudes.i"


// Extra c code in wrapper file
%{

// include relevant faudes headers
#include "corefaudes.h"
#include "pex_include.h"

// use libfaudes namespace
using namespace faudes;

// access mini help system
extern void luafaudes_dict_insert_topic(const std::string& topic, const std::string& entry);
extern void luafaudes_dict_insert_entry(const std::string& topic, const std::string& key, const std::string& entry);

%}


// Extra Lua functions: copy to faudes name space
%luacode {
-- Copy example to faudes name space
for k,v in pairs(example) do faudes[k]=v end
}


// Extra Lua functions: debugging
#ifdef FAUDES_DEBUG_LUABINDINGS
%luacode {
-- Say hello
print("libFAUDES: example plugin")
}
#endif


/*
**************************************************
**************************************************
**************************************************

Example interface: plugin function(s)

**************************************************
**************************************************
**************************************************
*/

// tell SWIG about my one and only function
// void AlternativeAccessible(vGenerator& rGen);


// add entry to mini help system: introduce new topic "example"
SwigHelpTopic("Example","Example plugin function");

// add entry to mini help system: introduce my function 
// SwigHelpEntry("Example","Functions", " AlternativeAccessible(gen)");
